/**
 * Copyright (c) 2015 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "nrf_drv_spi.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <string.h>
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "SEGGER_RTT.h"

#include "XL372.h"
#include "adxl372_spi.h"

#define SPI_INSTANCE  0
static const nrf_drv_spi_t    m_spi0_master = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);
static volatile bool spi_xfer_done;

#define TEST_STRING "Nordic"
static uint8_t       m_tx_buf[] = TEST_STRING;           /**< TX buffer. */
static const uint8_t m_tx_length = sizeof(m_tx_buf);
static uint8_t       m_rx_buf[sizeof(TEST_STRING) + 1];    /**< RX buffer. */
static const uint8_t m_rx_length = sizeof(m_rx_buf);        /**< Transfer length. */


/**
 * @brief SPI user event handler.
 * @param event
 */
void spi_event_handler(nrf_drv_spi_evt_t const *p_event, void *p_context)
{
    char str[128], str1[16];
    short i;

    spi_xfer_done = true;
    NRF_LOG_INFO("Transfer completed.");
    switch (p_event->type)
    {
        // The One and Only SPI event!
        case NRF_DRV_SPI_EVENT_DONE:
        {
            if(m_rx_buf[0] != 0xCC)   // High Probability that something changed
            {
                sprintf (str, " rxBuf Changed: ");
                for (i = 0; i < m_rx_length; i++)
                {
                    sprintf (str1, "[%d]= 0x%x, ", i, m_rx_buf[i]);
                    strcat (str, str1);
                }

                NRF_LOG_INFO("%s", str);
            }
            else
            {
                NRF_LOG_INFO(" rxBuf apparently didn't change");
            }
        }
    }

}


/**@brief Function for initializing a SPI master driver.
 *
 * @param[in] p_instance    Pointer to SPI master driver instance.
 * @param[in] lsb           Bits order LSB if true, MSB if false.
 */
static void spi_master_init(nrf_drv_spi_t const *p_instance)
{
    uint32_t              err_code;

    nrf_drv_spi_config_t  config;
    
    config.ss_pin       = NRF_DRV_SPI_PIN_NOT_USED;
    config.irq_priority = APP_IRQ_PRIORITY_LOW;
    config.orc          = 0xFF;
    config.frequency    = NRF_DRV_SPI_FREQ_125K;
    config.mode         = NRF_DRV_SPI_MODE_0;  // Corresponds to the datasheet specification CPHA = CPOL = 0
    config.bit_order    = NRF_DRV_SPI_BIT_ORDER_MSB_FIRST;  // Put it out the way you put it in
    
    #if (SPI0_ENABLED == 1)
    if (p_instance == &m_spi0_master)
    {
        config.sck_pin  = SPI0_SCK_PIN;
        config.mosi_pin = SPI0_MOSI_PIN;
        config.miso_pin = SPI0_MISO_PIN;
        err_code = nrf_drv_spi_init(p_instance, &config, spi_event_handler, NULL);
        
        APP_ERROR_CHECK(err_code);
    }
    else
    {
        // Do nothing
    }
    #endif // (SPI0_ENABLED == 1)

    // Set the SPI Chip Select pin as output and initial its state to high
    nrf_gpio_cfg_output(SPI0_SS_PIN);
    nrf_gpio_pin_set(SPI0_SS_PIN);
    
}


int main(void)
{
    char        c = 0;
    uint8_t     address = 0;
    uint32_t    err_code = 0;

    bsp_board_init(BSP_INIT_LEDS);

    err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);
    NRF_LOG_DEFAULT_BACKENDS_INIT();

    spi_master_init(&m_spi0_master);
    ADXL372_SPI_init(&m_spi0_master);

    NRF_LOG_INFO("SPI example started.");
    NRF_LOG_FLUSH();

    while (1)
    {
       
        // Reset rx & tx buffers and transfer done flag
        memset(m_rx_buf, 0, m_rx_length);
        memset(m_tx_buf, 0, m_tx_length);
        spi_xfer_done = false;

        // Artificial value that has to change on the transmission of the
        // register address. That byte in will produce a garbage byte out.
        // Betting on a low probability that the garbage will be 0xCC, so
        // I can use this as a flag in the spi_event_handler.
        // If the first byte of the received buffer isn't this value then
        // something did something somewhere along the process.
        m_rx_buf[0] = 0xCC;

        NRF_LOG_INFO("Hit a key to do a nrf_drv_spi_transfer.");
        NRF_LOG_FLUSH();
        c = SEGGER_RTT_WaitKey(); // will block until you're ready to try
        switch (c)
        {
            case 'q':
            {
                address = XL372_DEVID_AD;
                break;
            }
            case 'w':
            {
                address = XL372_DEVID_MST;
                break;
            }
            case 'e':
            {
                address = XL372_PARTID;
                break;
            }
            case 'r':
            {
                address = XL372_REVID;
                break;
            }
            default:
                spi_xfer_done = true;
        }
        if (!spi_xfer_done)
        {
            ADXL372_SPI_readRegister(address, m_rx_buf, 2);

            while (!spi_xfer_done)
            {
                __WFE();
            }

            NRF_LOG_FLUSH();

            bsp_board_led_invert(BSP_BOARD_LED_0);
            nrf_delay_ms(200);
        }
        else
            NRF_LOG_INFO("Wrong Key!");

    }
}
